@@ -5,22 +5,55 @@ from uritools import urisplit
 
 class UriSplitTest(unittest.TestCase):
 
-    def check(self, uri, split):
+    def check(self, uri, parts):
         result = urisplit(uri)
-        self.assertEqual(result, split)
+        self.assertEqual(result, parts)
         self.assertEqual(result.geturi(), uri)
+        for r, s in zip(result, parts):
+            self.assertIsInstance(r, type(s))
+        self.assertIsInstance(result.geturi(), type(uri))
 
     def test_rfc3986_3(self):
         """urisplit test cases from [RFC3986] 3. Syntax Components"""
+        self.check(
+            b'foo://example.com:8042/over/there?name=ferret#nose',
+            (b'foo', b'example.com:8042', b'/over/there', b'name=ferret', b'nose')
+        )
+        self.check(
+            u'foo://example.com:8042/over/there?name=ferret#nose',
+            (u'foo', u'example.com:8042', u'/over/there', u'name=ferret', u'nose')
+        )
+        self.check(
+            b'urn:example:animal:ferret:nose',
+            (b'urn', None, b'example:animal:ferret:nose', None, None)
+        )
+        self.check(
+            u'urn:example:animal:ferret:nose',
+            (u'urn', None, u'example:animal:ferret:nose', None, None)
+        )
 
-        self.check(
-            'foo://example.com:8042/over/there?name=ferret#nose',
-            ('foo', 'example.com:8042', '/over/there', 'name=ferret', 'nose')
-        )
-        self.check(
-            'urn:example:animal:ferret:nose',
-            ('urn', None, 'example:animal:ferret:nose', None, None)
-        )
+    def test_pathologic(self):
+        """urisplit edge cases"""
+        for uri, parts in [
+            ('', (None, None, '', None, None)),
+            (':', (None, None, ':', None, None)),
+            (':/', (None, None, ':/', None, None)),
+            ('://', (None, None, '://', None, None)),
+            ('://?', (None, None, '://', '', None)),
+            ('://#', (None, None, '://', None, '')),
+            ('://?#', (None, None, '://', '', '')),
+            ('//', (None, '', '', None, None)),
+            ('///', (None, '', '/', None, None)),
+            ('//?', (None, '', '', '', None)),
+            ('//#', (None, '', '', None, '')),
+            ('//?#', (None, '', '', '', '')),
+            ('?', (None, None, '', '', None)),
+            ('??', (None, None, '', '?', None)),
+            ('?#', (None, None, '', '', '')),
+            ('#', (None, None, '', None, '')),
+            ('##', (None, None, '', None, '#')),
+        ]:
+            self.check(uri, parts)
 
     def test_urlparse_roundtrips(self):
         """urlparse roundtrip test cases"""
@@ -59,14 +92,17 @@ class UriSplitTest(unittest.TestCase):
         uri = "HTTP://WWW.PYTHON.ORG/doc/#frag"
         p = urisplit(uri)
         #self.assertEqual(p.scheme, "http")
+        self.assertEqual(p.scheme.lower(), "http")
         self.assertEqual(p.authority, "WWW.PYTHON.ORG")
         self.assertEqual(p.path, "/doc/")
         self.assertEqual(p.query, None)
         self.assertEqual(p.fragment, "frag")
         #self.assertEqual(p.username, None)
         #self.assertEqual(p.password, None)
-        #self.assertEqual(p.hostname, "www.python.org")
-        #self.assertEqual(p.port, None)
+        self.assertEqual(p.userinfo, None)
+        #self.assertEqual(p.host, "www.python.org")
+        self.assertEqual(p.host.lower(), "www.python.org")
+        self.assertEqual(p.port, None)
         self.assertEqual(p.geturi(), uri)
 
         uri = "http://User:Pass@www.python.org:080/doc/?query=yes#frag"
@@ -78,8 +114,9 @@ class UriSplitTest(unittest.TestCase):
         self.assertEqual(p.fragment, "frag")
         #self.assertEqual(p.username, "User")
         #self.assertEqual(p.password, "Pass")
-        #self.assertEqual(p.hostname, "www.python.org")
-        #self.assertEqual(p.port, 80)
+        self.assertEqual(p.userinfo, 'User:Pass')
+        self.assertEqual(p.host, "www.python.org")
+        self.assertEqual(p.port, 80)
         self.assertEqual(p.geturi(), uri)
 
         uri = "http://User@example.com:Pass@www.python.org:080/doc/?query=yes#frag"
@@ -91,8 +128,9 @@ class UriSplitTest(unittest.TestCase):
         self.assertEqual(p.fragment, "frag")
         #self.assertEqual(p.username, "User@example.com")
         #self.assertEqual(p.password, "Pass")
-        #self.assertEqual(p.hostname, "www.python.org")
-        #self.assertEqual(p.port, 80)
+        self.assertEqual(p.userinfo, 'User@example.com:Pass')
+        self.assertEqual(p.host, "www.python.org")
+        self.assertEqual(p.port, 80)
         self.assertEqual(p.geturi(), uri)
 
         uri = "sip:alice@atlanta.com;maddr=239.255.255.1;ttl=15"
@@ -100,6 +138,7 @@ class UriSplitTest(unittest.TestCase):
         self.assertEqual(p.authority, None)
         #self.assertEqual(p.username, None)
         #self.assertEqual(p.password, None)
-        #self.assertEqual(p.hostname, None)
-        #self.assertEqual(p.port, None)
+        self.assertEqual(p.userinfo, None)
+        self.assertEqual(p.host, None)
+        self.assertEqual(p.port, None)
         self.assertEqual(p.geturi(), uri)
